@@ -3,6 +3,7 @@ package ptp
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"strings"
@@ -20,8 +21,10 @@ import (
 	ptpv1 "github.com/openshift/ptp-operator/pkg/apis/ptp/v1"
 
 	. "github.com/openshift/ptp-operator/test/utils"
+	"github.com/openshift/ptp-operator/test/utils/clean"
 	"github.com/openshift/ptp-operator/test/utils/client"
 	testclient "github.com/openshift/ptp-operator/test/utils/client"
+	"github.com/openshift/ptp-operator/test/utils/discovery"
 	"github.com/openshift/ptp-operator/test/utils/execute"
 	"github.com/openshift/ptp-operator/test/utils/nodes"
 	"github.com/openshift/ptp-operator/test/utils/pods"
@@ -57,13 +60,13 @@ var _ = Describe("[ptp]", func() {
 		// Setup verification
 		It("Should check that all nodes are running at least one replica of linuxptp-daemon", func() {
 			By("Getting list of nodes")
-			nodes, err := client.Client.Nodes().List(metav1.ListOptions{})
+			nodes, err := client.Client.Nodes().List(context.Background(), metav1.ListOptions{})
 			Expect(err).NotTo(HaveOccurred())
 			By("Checking number of nodes")
 			Expect(len(nodes.Items)).To(BeNumerically(">", 0), "number of nodes should be more than 0")
 
 			By("Get daemonsets collection for the namespace " + PtpLinuxDaemonNamespace)
-			ds, err := client.Client.DaemonSets(PtpLinuxDaemonNamespace).List(metav1.ListOptions{})
+			ds, err := client.Client.DaemonSets(PtpLinuxDaemonNamespace).List(context.Background(), metav1.ListOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			Expect(len(ds.Items)).To(BeNumerically(">", 0), "no damonsets found in the namespace "+PtpLinuxDaemonNamespace)
 			By("Checking number of scheduled instances")
@@ -72,7 +75,7 @@ var _ = Describe("[ptp]", func() {
 		// Setup verification
 		It("Should check that operator is deployed", func() {
 			By("Getting deployment " + PtpOperatorDeploymentName)
-			dep, err := client.Client.Deployments(PtpLinuxDaemonNamespace).Get(PtpOperatorDeploymentName, metav1.GetOptions{})
+			dep, err := client.Client.Deployments(PtpLinuxDaemonNamespace).Get(context.Background(), PtpOperatorDeploymentName, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			By("Checking availability of the deployment")
 			for _, c := range dep.Status.Conditions {
@@ -85,85 +88,45 @@ var _ = Describe("[ptp]", func() {
 
 	Describe("PTP e2e tests", func() {
 		var ptpRunningPods []v1core.Pod
+		var masterNodeLabel, slaveNodeLabel string
+		discoveryFailed := false
+		var masterProfile, slaveProfile string
 
 		execute.BeforeAll(func() {
-			ptpconfigList, err := client.Client.PtpConfigs(PtpLinuxDaemonNamespace).List(metav1.ListOptions{})
-			Expect(err).ToNot(HaveOccurred())
 
-			for _, ptpConfig := range ptpconfigList.Items {
-				if ptpConfig.Name == PtpGrandMasterPolicyName || ptpConfig.Name == PtpSlavePolicyName {
-					err = client.Client.PtpConfigs(PtpLinuxDaemonNamespace).Delete(ptpConfig.Name, &metav1.DeleteOptions{})
-					Expect(err).ToNot(HaveOccurred())
-				}
+			if !discovery.Enabled() {
+				configurePTP()
+			}
+			masterConfigs, slaveConfigs := discoveryPTPConfiguration(PtpLinuxDaemonNamespace)
+
+			if len(masterConfigs) != 0 {
+				masterRes := checkPtpProfileLabels(masterConfigs)
+				masterProfile = masterRes.profileName
+				masterNodeLabel = masterRes.label
+			}
+			if len(slaveConfigs) == 0 {
+				discoveryFailed = true
+				return
+			}
+			slaveRes := checkPtpProfileLabels(slaveConfigs)
+			slaveProfile = slaveRes.profileName
+			slaveNodeLabel = slaveRes.label
+			if slaveNodeLabel == "" {
+				discoveryFailed = true
+				return
 			}
 
-			nodeList, err := client.Client.Nodes().List(metav1.ListOptions{LabelSelector: fmt.Sprintf("%s=", PtpGrandmasterNodeLabel)})
-			Expect(err).ToNot(HaveOccurred())
-			for _, node := range nodeList.Items {
-				delete(node.Labels, PtpGrandmasterNodeLabel)
-				_, err = client.Client.Nodes().Update(&node)
-				Expect(err).ToNot(HaveOccurred())
-			}
-
-			nodeList, err = client.Client.Nodes().List(metav1.ListOptions{LabelSelector: fmt.Sprintf("%s=", PtpSlaveNodeLabel)})
-			Expect(err).ToNot(HaveOccurred())
-			for _, node := range nodeList.Items {
-				delete(node.Labels, PtpSlaveNodeLabel)
-				_, err = client.Client.Nodes().Update(&node)
-				Expect(err).ToNot(HaveOccurred())
-			}
-
-			ptpNodes, err := nodes.GetNodeTopology(client.Client)
-			Expect(err).ToNot(HaveOccurred())
-			Expect(len(ptpNodes)).To(BeNumerically(">", 1), "need at least two nodes with ptp capable nics")
-
-			By("Labeling the grandmaster node")
-			ptpGrandMasterNode := ptpNodes[0]
-			ptpGrandMasterNode.NodeObject, err = nodes.LabelNode(ptpGrandMasterNode.NodeName, PtpGrandmasterNodeLabel, "")
-			Expect(err).ToNot(HaveOccurred())
-
-			By("Labeling the slave node")
-			ptpSlaveNode := ptpNodes[1]
-			ptpSlaveNode.NodeObject, err = nodes.LabelNode(ptpSlaveNode.NodeName, PtpSlaveNodeLabel, "")
-			Expect(err).ToNot(HaveOccurred())
-
-			By("Creating the policy for the grandmaster node")
-			err = createConfig(PtpGrandMasterPolicyName,
-				ptpGrandMasterNode.InterfaceList[0],
-				"",
-				"-a -r -r",
-				PtpGrandmasterNodeLabel,
-				pointer.Int64Ptr(5))
-			Expect(err).ToNot(HaveOccurred())
-
-			By("Creating the policy for the slave node")
-			err = createConfig(PtpSlavePolicyName,
-				ptpSlaveNode.InterfaceList[0],
-				"-s",
-				"-a -r",
-				PtpSlaveNodeLabel,
-				pointer.Int64Ptr(5))
-			Expect(err).ToNot(HaveOccurred())
-
-			By("Restart the linuxptp-daemon pods")
-			ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
-			Expect(err).ToNot(HaveOccurred())
-			for _, pod := range ptpPods.Items {
-				err = client.Client.Pods(PtpLinuxDaemonNamespace).Delete(pod.Name, &metav1.DeleteOptions{GracePeriodSeconds: pointer.Int64Ptr(0)})
-				Expect(err).ToNot(HaveOccurred())
-			}
-
-			daemonset, err := client.Client.DaemonSets(PtpLinuxDaemonNamespace).Get(PtpDaemonsetName, metav1.GetOptions{})
+			daemonset, err := client.Client.DaemonSets(PtpLinuxDaemonNamespace).Get(context.Background(), PtpDaemonsetName, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			expectedNumber := daemonset.Status.DesiredNumberScheduled
 			Eventually(func() int32 {
-				daemonset, err = client.Client.DaemonSets(PtpLinuxDaemonNamespace).Get(PtpDaemonsetName, metav1.GetOptions{})
+				daemonset, err = client.Client.DaemonSets(PtpLinuxDaemonNamespace).Get(context.Background(), PtpDaemonsetName, metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
 				return daemonset.Status.NumberReady
 			}, 2*time.Minute, 2*time.Second).Should(Equal(expectedNumber))
 
 			Eventually(func() int {
-				ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
+				ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(context.Background(), metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
 				Expect(err).ToNot(HaveOccurred())
 				return len(ptpPods.Items)
 			}, 2*time.Minute, 2*time.Second).Should(Equal(int(expectedNumber)))
@@ -171,12 +134,15 @@ var _ = Describe("[ptp]", func() {
 
 		Context("PTP Interfaces discovery", func() {
 			BeforeEach(func() {
+				if discoveryFailed {
+					Skip("Failed to find a valid ptp slave configuration")
+				}
 				ptpRunningPods = []v1core.Pod{}
-				ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
+				ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(context.Background(), metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
 				Expect(err).NotTo(HaveOccurred())
 				Expect(len(ptpPods.Items)).To(BeNumerically(">", 0), fmt.Sprint("linuxptp-daemon is not deployed on cluster"))
 				for _, pod := range ptpPods.Items {
-					if podRole(pod, PtpSlaveNodeLabel) || podRole(pod, PtpGrandmasterNodeLabel) {
+					if podRole(pod, slaveNodeLabel) || podRole(pod, masterNodeLabel) {
 						waitUntilLogIsDetected(pod, 3*time.Minute, "Profile Name:")
 						ptpRunningPods = append(ptpRunningPods, pod)
 					}
@@ -210,16 +176,20 @@ var _ = Describe("[ptp]", func() {
 
 			// 25733
 			It("PTP daemon apply match rule based on nodeLabel", func() {
-				profileSlave := "Profile Name: test-slave"
-				profileMaster := "Profile Name: test-grandmaster"
+				profileSlave := fmt.Sprintf("Profile Name: %s", slaveProfile)
+				profileMaster := ""
+				if !discovery.Enabled() {
+					profileMaster = fmt.Sprintf("Profile Name: %s", masterProfile)
+				}
+
 				for _, pod := range ptpRunningPods {
-					podLogs, err := pods.GetLog(&pod)
+					podLogs, err := pods.GetLog(&pod, PtpContainerName)
 					Expect(err).NotTo(HaveOccurred(), "Error to find needed log due to %s", err)
-					if podRole(pod, PtpSlaveNodeLabel) {
+					if podRole(pod, slaveNodeLabel) {
 						Expect(podLogs).Should(ContainSubstring(profileSlave),
 							fmt.Sprintf("Profile \"%s\" not found in pod's log %s", profileSlave, pod.Name))
 					}
-					if podRole(pod, PtpGrandmasterNodeLabel) {
+					if podRole(pod, masterNodeLabel) && !discovery.Enabled() {
 						Expect(podLogs).Should(ContainSubstring(profileMaster),
 							fmt.Sprintf("Profile \"%s\" not found in pod's log %s", profileSlave, pod.Name))
 					}
@@ -228,12 +198,15 @@ var _ = Describe("[ptp]", func() {
 
 			// 25738
 			It("Slave can sync to master", func() {
+				if masterNodeLabel == "" {
+					Skip("No nodes configured as ptp master found on the cluster.")
+				}
 				var masterID string
 				var slaveMasterID string
 				grandMaster := "assuming the grand master role"
 				for _, pod := range ptpRunningPods {
-					if podRole(pod, PtpGrandmasterNodeLabel) {
-						podLogs, err := pods.GetLog(&pod)
+					if podRole(pod, masterNodeLabel) {
+						podLogs, err := pods.GetLog(&pod, PtpContainerName)
 						Expect(err).NotTo(HaveOccurred(), "Error to find needed log due to %s", err)
 						Expect(podLogs).Should(ContainSubstring(grandMaster),
 							fmt.Sprintf("Log message \"%s\" not found in pod's log %s", grandMaster, pod.Name))
@@ -243,9 +216,10 @@ var _ = Describe("[ptp]", func() {
 							}
 						}
 					}
-					if podRole(pod, PtpSlaveNodeLabel) {
-						podLogs, err := pods.GetLog(&pod)
+					if podRole(pod, slaveNodeLabel) {
+						podLogs, err := pods.GetLog(&pod, PtpContainerName)
 						Expect(err).NotTo(HaveOccurred(), "Error to find needed log due to %s", err)
+
 						for _, line := range strings.Split(podLogs, "\n") {
 							if strings.Contains(line, "new foreign master") {
 								slaveMasterID = strings.Split(line, " ")[6]
@@ -261,19 +235,22 @@ var _ = Describe("[ptp]", func() {
 			//25743
 			It("Can provide a profile with higher priority", func() {
 				var testPtpPod v1core.Pod
+				if discovery.Enabled() {
+					Skip("Skipping because adding a different profile")
+				}
 
 				By("Creating a config with higher priority", func() {
-					ptpConfigSlave, err := client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Get(PtpSlavePolicyName, metav1.GetOptions{})
+					ptpConfigSlave, err := client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Get(context.Background(), PtpSlavePolicyName, metav1.GetOptions{})
 					Expect(err).NotTo(HaveOccurred())
-					nodes, err := client.Client.Nodes().List(metav1.ListOptions{
-						LabelSelector: PtpSlaveNodeLabel,
+					nodes, err := client.Client.Nodes().List(context.Background(), metav1.ListOptions{
+						LabelSelector: slaveNodeLabel,
 					})
 					Expect(err).NotTo(HaveOccurred())
 					Expect(len(nodes.Items)).To(BeNumerically(">", 0),
-						fmt.Sprintf("PTP Nodes with label %s are not deployed on cluster", PtpSlaveNodeLabel))
+						fmt.Sprintf("PTP Nodes with label %s are not deployed on cluster", slaveNodeLabel))
 
 					ptpConfigTest := mutateProfile(*ptpConfigSlave, PtpSlavePolicyName, nodes.Items[0].Name)
-					_, err = client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Create(ptpConfigTest)
+					_, err = client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Create(context.Background(), ptpConfigTest, metav1.CreateOptions{})
 					Expect(err).NotTo(HaveOccurred())
 
 					testPtpPod, err = getPtpPodOnNode(nodes.Items[0].Name)
@@ -288,10 +265,10 @@ var _ = Describe("[ptp]", func() {
 				})
 
 				By("Deleting the test profile", func() {
-					err := client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Delete("test", &metav1.DeleteOptions{})
+					err := client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Delete(context.Background(), "test", metav1.DeleteOptions{})
 					Expect(err).NotTo(HaveOccurred())
 					Eventually(func() bool {
-						_, err := client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Get("test", metav1.GetOptions{})
+						_, err := client.Client.PtpV1Interface.PtpConfigs("openshift-ptp").Get(context.Background(), "test", metav1.GetOptions{})
 						return errors.IsNotFound(err)
 					}, 1*time.Minute, 1*time.Second).Should(BeTrue(), "Could not delete the test profile")
 				})
@@ -301,12 +278,185 @@ var _ = Describe("[ptp]", func() {
 				})
 			})
 		})
-	})
 
+		Context("PTP metric is present", func() {
+			BeforeEach(func() {
+				if discoveryFailed {
+					Skip("Failed to find a valid ptp slave configuration")
+				}
+				ptpRunningPods = []v1core.Pod{}
+				ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(context.Background(), metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
+				Expect(err).NotTo(HaveOccurred())
+				Expect(len(ptpPods.Items)).To(BeNumerically(">", 0), fmt.Sprint("linuxptp-daemon is not deployed on cluster"))
+				for _, pod := range ptpPods.Items {
+					if podRole(pod, slaveNodeLabel) || podRole(pod, masterNodeLabel) {
+						waitUntilLogIsDetected(pod, 3*time.Minute, "Profile Name:")
+						ptpRunningPods = append(ptpRunningPods, pod)
+					}
+				}
+				Expect(len(ptpRunningPods)).To(BeNumerically(">=", 2), fmt.Sprint("Fail to detect PTP slave/master pods on Cluster"))
+			})
+
+			// 27324
+			It("on slave", func() {
+				slavePodDetected := false
+				for _, pod := range ptpRunningPods {
+					if podRole(pod, slaveNodeLabel) {
+						Eventually(func() string {
+							buf, _ := pods.ExecCommand(client.Client, pod, PtpContainerName, []string{"curl", "127.0.0.1:9091/metrics"})
+							return buf.String()
+						}, 5*time.Minute, 5*time.Second).Should(ContainSubstring("openshift_ptp_max_offset_from_master"),
+							fmt.Sprint("Time metrics are not detected"))
+						slavePodDetected = true
+						break
+					}
+				}
+				Expect(slavePodDetected).ToNot(BeFalse(), "No slave pods detected")
+			})
+		})
+	})
 })
 
+func configurePTP() {
+	err := clean.All()
+	Expect(err).ToNot(HaveOccurred())
+
+	ptpNodes, err := nodes.PtpEnabled(client.Client)
+	Expect(err).ToNot(HaveOccurred())
+	Expect(len(ptpNodes)).To(BeNumerically(">", 1), "need at least two nodes with ptp capable nics")
+
+	By("Labeling the grandmaster node")
+	ptpGrandMasterNode := ptpNodes[0]
+	ptpGrandMasterNode.NodeObject, err = nodes.LabelNode(ptpGrandMasterNode.NodeName, PtpGrandmasterNodeLabel, "")
+	Expect(err).ToNot(HaveOccurred())
+
+	By("Labeling the slave node")
+	ptpSlaveNode := ptpNodes[1]
+	ptpSlaveNode.NodeObject, err = nodes.LabelNode(ptpSlaveNode.NodeName, PtpSlaveNodeLabel, "")
+	Expect(err).ToNot(HaveOccurred())
+
+	for _, gmInterface := range ptpGrandMasterNode.InterfaceList {
+		for _, slaveInterface := range ptpSlaveNode.InterfaceList {
+			clean.Configs()
+			fmt.Printf("Validating interface %s for grandmaster, %s for slave\n", gmInterface, slaveInterface)
+			By("Creating the policy for the grandmaster node")
+			err = createConfig(PtpGrandMasterPolicyName,
+				gmInterface,
+				"",
+				"-a -r -r",
+				PtpGrandmasterNodeLabel,
+				pointer.Int64Ptr(5))
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Creating the policy for the slave node")
+			err = createConfig(PtpSlavePolicyName,
+				slaveInterface,
+				"-s",
+				"-a -r",
+				PtpSlaveNodeLabel,
+				pointer.Int64Ptr(5))
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Restart the linuxptp-daemon pods")
+			ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(context.Background(), metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
+			Expect(err).ToNot(HaveOccurred())
+			for _, pod := range ptpPods.Items {
+				err = client.Client.Pods(PtpLinuxDaemonNamespace).Delete(context.Background(), pod.Name, metav1.DeleteOptions{GracePeriodSeconds: pointer.Int64Ptr(0)})
+				Expect(err).ToNot(HaveOccurred())
+			}
+
+			daemonset, err := client.Client.DaemonSets(PtpLinuxDaemonNamespace).Get(context.Background(), PtpDaemonsetName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			expectedNumber := daemonset.Status.DesiredNumberScheduled
+			Eventually(func() int32 {
+				daemonset, err = client.Client.DaemonSets(PtpLinuxDaemonNamespace).Get(context.Background(), PtpDaemonsetName, metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				return daemonset.Status.NumberReady
+			}, 2*time.Minute, 2*time.Second).Should(Equal(expectedNumber))
+
+			Eventually(func() int {
+				ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(context.Background(), metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
+				Expect(err).ToNot(HaveOccurred())
+				return len(ptpPods.Items)
+			}, 2*time.Minute, 2*time.Second).Should(Equal(int(expectedNumber)))
+
+			for i := 0; i < 60; i++ {
+				time.Sleep(1 * time.Second)
+				ptpPods, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(context.Background(),
+					metav1.ListOptions{LabelSelector: "app=linuxptp-daemon", FieldSelector: fmt.Sprintf("spec.nodeName=%s", ptpSlaveNode.NodeName)})
+				Expect(err).ToNot(HaveOccurred())
+				Expect(len(ptpPods.Items)).To(Equal(1))
+
+				logs, _ := pods.GetLog(&ptpPods.Items[0], PtpContainerName)
+				if strings.Contains(logs, "new foreign master") {
+					fmt.Printf("Found valid PTP Configuration, using %s for master, %s for slave after %d seconds", gmInterface, slaveInterface, i)
+					return
+				}
+			}
+		}
+	}
+	fmt.Println("Did not find a valid PTP configuration")
+
+}
+
+// Returns the slave node label to be used in the test
+func discoveryPTPConfiguration(namespace string) ([]ptpv1.PtpConfig, []ptpv1.PtpConfig) {
+	var masters []ptpv1.PtpConfig
+	var slaves []ptpv1.PtpConfig
+
+	configList, err := client.Client.PtpConfigs(namespace).List(context.Background(), metav1.ListOptions{})
+	Expect(err).ToNot(HaveOccurred())
+	for _, config := range configList.Items {
+		for _, profile := range config.Spec.Profile {
+			if isPtpMaster(*profile.Ptp4lOpts, *profile.Phc2sysOpts) {
+				masters = append(masters, config)
+			}
+			if isPtpSlave(*profile.Ptp4lOpts, *profile.Phc2sysOpts) {
+				slaves = append(slaves, config)
+			}
+		}
+	}
+	return masters, slaves
+}
+
+type ptpDiscoveryRes struct {
+	label       string
+	profileName string
+}
+
+func checkPtpProfileLabels(configs []ptpv1.PtpConfig) ptpDiscoveryRes {
+	for _, config := range configs {
+		for _, recommend := range config.Spec.Recommend {
+			for _, match := range recommend.Match {
+				label := *match.NodeLabel
+				nodeCount := checkLabeledNodesExists(label)
+
+				if nodeCount > 0 {
+					return ptpDiscoveryRes{label, config.Name}
+				}
+			}
+		}
+	}
+	return ptpDiscoveryRes{"", ""}
+}
+
+func checkLabeledNodesExists(label string) int {
+	nodeList, err := client.Client.Nodes().List(context.Background(), metav1.ListOptions{LabelSelector: fmt.Sprintf("%s=", label)})
+	Expect(err).ToNot(HaveOccurred())
+	return len(nodeList.Items)
+}
+
+func isPtpSlave(ptp4lOpts string, phc2sysOpts string) bool {
+	return strings.Contains(ptp4lOpts, "-s") && strings.Count(phc2sysOpts, "-a") == 1 && strings.Count(phc2sysOpts, "-r") == 1
+
+}
+
+func isPtpMaster(ptp4lOpts string, phc2sysOpts string) bool {
+	return !strings.Contains(ptp4lOpts, "-s") && strings.Count(phc2sysOpts, "-a") == 1 && strings.Count(phc2sysOpts, "-r") == 2
+}
+
 func podRole(runningPod v1core.Pod, role string) bool {
-	nodeList, err := client.Client.Nodes().List(metav1.ListOptions{
+	nodeList, err := client.Client.Nodes().List(context.Background(), metav1.ListOptions{
 		LabelSelector: role,
 	})
 	Expect(err).NotTo(HaveOccurred())
@@ -344,7 +494,7 @@ func isPTPEnabled(ethToolOutput *bytes.Buffer) bool {
 func ptpDiscoveredInterfaceList(path string) []string {
 	var ptpInterfaces []string
 	var nodePtpDevice ptpv1.NodePtpDevice
-	fg, err := client.Client.CoreV1Interface.RESTClient().Get().AbsPath(path).DoRaw()
+	fg, err := client.Client.CoreV1Interface.RESTClient().Get().AbsPath(path).DoRaw(context.Background())
 	Expect(err).ToNot(HaveOccurred())
 
 	err = json.Unmarshal(fg, &nodePtpDevice)
@@ -372,13 +522,13 @@ func mutateProfile(profile ptpv1.PtpConfig, profileName string, nodeName string)
 
 func waitUntilLogIsDetected(pod v1core.Pod, timeout time.Duration, neededLog string) {
 	Eventually(func() string {
-		logs, _ := pods.GetLog(&pod)
+		logs, _ := pods.GetLog(&pod, PtpContainerName)
 		return logs
 	}, timeout, 1*time.Second).Should(ContainSubstring(neededLog), fmt.Sprintf("Timeout to detect log \"%s\" in pod \"%s\"", neededLog, pod.Name))
 }
 
 func getPtpPodOnNode(nodeName string) (v1core.Pod, error) {
-	runningPod, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
+	runningPod, err := client.Client.Pods(PtpLinuxDaemonNamespace).List(context.Background(), metav1.ListOptions{LabelSelector: "app=linuxptp-daemon"})
 	Expect(err).NotTo(HaveOccurred(), fmt.Sprint("Error to get list of pods by label: app=linuxptp-daemon"))
 	Expect(len(runningPod.Items)).To(BeNumerically(">", 0), fmt.Sprint("PTP pods are  not deployed on cluster"))
 	for _, pod := range runningPod.Items {
@@ -393,7 +543,7 @@ func getPtpPodOnNode(nodeName string) (v1core.Pod, error) {
 func getMasterSlaveAttachedInterfaces(pod v1core.Pod) []string {
 	var IntList []string
 	Eventually(func() error {
-		stdout, err := pods.ExecCommand(client.Client, pod, []string{"ls", "/sys/class/net/"})
+		stdout, err := pods.ExecCommand(client.Client, pod, PtpContainerName, []string{"ls", "/sys/class/net/"})
 		if err != nil {
 			return err
 		}
@@ -425,7 +575,7 @@ func getPtpMasterSlaveAttachedInterfaces(pod v1core.Pod) []string {
 
 		// Get readlink status
 		Eventually(func() error {
-			stdout, err = pods.ExecCommand(client.Client, pod, []string{"readlink", "-f", fmt.Sprintf("/sys/class/net/%s", interf)})
+			stdout, err = pods.ExecCommand(client.Client, pod, PtpContainerName, []string{"readlink", "-f", fmt.Sprintf("/sys/class/net/%s", interf)})
 			if err != nil {
 				return err
 			}
@@ -458,7 +608,7 @@ func getPtpMasterSlaveAttachedInterfaces(pod v1core.Pod) []string {
 		// Check if this is a virtual function
 		Eventually(func() error {
 			// If the physfn doesn't exist this means the interface is not a virtual function so we ca add it to the list
-			stdout, err = pods.ExecCommand(client.Client, pod, []string{"ls", fmt.Sprintf("/sys/bus/pci/devices/%s/physfn", PCIAddr)})
+			stdout, err = pods.ExecCommand(client.Client, pod, PtpContainerName, []string{"ls", fmt.Sprintf("/sys/bus/pci/devices/%s/physfn", PCIAddr)})
 			if err != nil {
 				if strings.Contains(stdout.String(), "No such file or directory") {
 					return nil
@@ -480,7 +630,7 @@ func getPtpMasterSlaveAttachedInterfaces(pod v1core.Pod) []string {
 		}
 
 		Eventually(func() error {
-			stdout, err = pods.ExecCommand(client.Client, pod, []string{"ethtool", "-T", interf})
+			stdout, err = pods.ExecCommand(client.Client, pod, PtpContainerName, []string{"ethtool", "-T", interf})
 			if stdout.String() == "" {
 				return fmt.Errorf("empty response from pod retrying")
 			}
@@ -509,7 +659,7 @@ func getPtpMasterSlaveAttachedInterfaces(pod v1core.Pod) []string {
 func replaceTestPod(pod v1core.Pod, timeout time.Duration) (v1core.Pod, error) {
 	var newPod v1core.Pod
 
-	err := client.Client.Pods(PtpLinuxDaemonNamespace).Delete(pod.Name, &metav1.DeleteOptions{
+	err := client.Client.Pods(PtpLinuxDaemonNamespace).Delete(context.Background(), pod.Name, metav1.DeleteOptions{
 		GracePeriodSeconds: pointer.Int64Ptr(0)})
 	Expect(err).NotTo(HaveOccurred())
 
@@ -534,7 +684,7 @@ func getNonPtpMasterSlaveAttachedInterfaces(pod v1core.Pod) []string {
 	intList := getMasterSlaveAttachedInterfaces(pod)
 	for _, interf := range intList {
 		Eventually(func() error {
-			stdout, err = pods.ExecCommand(client.Client, pod, []string{"ethtool", "-T", interf})
+			stdout, err = pods.ExecCommand(client.Client, pod, PtpContainerName, []string{"ethtool", "-T", interf})
 			if err != nil && !strings.Contains(stdout.String(), "No such device") {
 				return err
 			}
@@ -562,6 +712,6 @@ func createConfig(profileName, ifaceName, ptp4lOpts, phc2sysOpts, nodeLabel stri
 	policy := ptpv1.PtpConfig{ObjectMeta: metav1.ObjectMeta{Name: profileName, Namespace: PtpLinuxDaemonNamespace},
 		Spec: ptpv1.PtpConfigSpec{Profile: []ptpv1.PtpProfile{ptpProfile}, Recommend: []ptpv1.PtpRecommend{ptpRecommend}}}
 
-	_, err := client.Client.PtpConfigs(PtpLinuxDaemonNamespace).Create(&policy)
+	_, err := client.Client.PtpConfigs(PtpLinuxDaemonNamespace).Create(context.Background(), &policy, metav1.CreateOptions{})
 	return err
 }
